/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package arcgissamples.soe;

import static com.esri.arcgis.system.ServerUtilities.getServerLogger;
import static com.esri.arcgis.system.ServerUtilities.sendError;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import arcgissamples.soe.javatypes.BusStation;
import arcgissamples.soe.javatypes.Coordinates;

import com.esri.arcgis.carto.IFeatureLayer;
import com.esri.arcgis.carto.Map;
import com.esri.arcgis.carto.MapServer;
import com.esri.arcgis.carto.NetworkLayer;
import com.esri.arcgis.display.CancelTracker;
import com.esri.arcgis.geodatabase.DENetworkDataset;
import com.esri.arcgis.geodatabase.FeatureClass;
import com.esri.arcgis.geodatabase.GPMessages;
import com.esri.arcgis.geodatabase.IDatasetComponent;
import com.esri.arcgis.geodatabase.IDatasetComponentProxy;
import com.esri.arcgis.geodatabase.IFeature;
import com.esri.arcgis.geodatabase.IFeatureCursor;
import com.esri.arcgis.geodatabase.NetworkDataset;
import com.esri.arcgis.geodatabase.QueryFilter;
import com.esri.arcgis.geometry.IPoint;
import com.esri.arcgis.geometry.Point;
import com.esri.arcgis.geometry.Polygon;
import com.esri.arcgis.geometry.SpatialReferenceEnvironment;
import com.esri.arcgis.interop.AutomationException;
import com.esri.arcgis.interop.extn.ArcGISExtension;
import com.esri.arcgis.interop.extn.ServerObjectExtProperties;
import com.esri.arcgis.networkanalyst.INAContext;
import com.esri.arcgis.networkanalyst.INALayer;
import com.esri.arcgis.networkanalyst.INALocation;
import com.esri.arcgis.networkanalyst.INASolver;
import com.esri.arcgis.networkanalyst.INAStreetDirection;
import com.esri.arcgis.networkanalyst.INAStreetDirections;
import com.esri.arcgis.networkanalyst.INAStreetDirectionsContainer;
import com.esri.arcgis.networkanalyst.INamedSet;
import com.esri.arcgis.networkanalyst.NAClass;
import com.esri.arcgis.networkanalyst.NAContext;
import com.esri.arcgis.networkanalyst.NALayer;
import com.esri.arcgis.networkanalyst.NALocation;
import com.esri.arcgis.networkanalyst.NALocationFeature;
import com.esri.arcgis.networkanalyst.NALocator;
import com.esri.arcgis.networkanalyst.NARouteSolver;
import com.esri.arcgis.networkanalyst.NAStreetDirectionsAgent;
import com.esri.arcgis.networkanalyst.NamedSet;
import com.esri.arcgis.networkanalyst.esriNAObjectStatus;
import com.esri.arcgis.networkanalyst.esriNAOutputLineType;
import com.esri.arcgis.server.IServerObjectExtension;
import com.esri.arcgis.server.IServerObjectHelper;
import com.esri.arcgis.server.SOAPRequestHandler;
import com.esri.arcgis.server.json.JSONArray;
import com.esri.arcgis.server.json.JSONException;
import com.esri.arcgis.server.json.JSONObject;
import com.esri.arcgis.system.ILog;
import com.esri.arcgis.system.IRESTRequestHandler;
import com.esri.arcgis.system.ITrackCancel;
import com.esri.arcgis.system.ServerUtilities;
import com.esri.arcgis.system.StrArray;
import com.esri.arcgis.system.esriUnits;

@ArcGISExtension
@ServerObjectExtProperties(
		displayName = "Bus Services", 
		description = "Finds bus stations and directions in Portland, OR.", 
		defaultSOAPCapabilities = {"Access Bus Stations" }, 
		allSOAPCapabilities = {"Access Bus Stations", "Find Directions" },
		supportsMSD = false
)
public class BusServicesSOE extends SOAPRequestHandler implements IServerObjectExtension, IRESTRequestHandler, IBusService
{
	private static final long serialVersionUID = 1L;
	private IServerObjectHelper soHelper;
	private ILog serverLog;
	private MapServer mapServer;
	private FeatureClass busStationsFC;
	
	//NA Route Solver related objects
	private NetworkDataset nds = null;	
	private INASolver naSolver = null;
	private NAContext naContext = null;
	private int nameFieldIndex, sequenceFieldIndex,  statusFieldIndex;
	private NAClass stopsNAClass = null;
	private NALocator naLocator = null;

	public BusServicesSOE() throws Exception
	{
		super();
	}

	/****************************************************************************************************************************
	 * IServerObjectExtension methods: This is a mandatory interface that must be supported by all SOEs. This interface
	 * is used by the Server Object to manage the lifetime of the SOE and includes two methods: init() and shutdown().
	 * The Server Object cocreates the SOE and calls the init() method handing it a back reference to the Server Object
	 * via the Server Object Helper argument. The Server Object Helper implements a weak reference on the Server Object.
	 * The extension can keep a strong reference on the Server Object Helper (for example, in a member variable) but
	 * should not keep a strong reference on the Server Object. The log entries are merely informative and completely
	 * optional.
	 ****************************************************************************************************************************/
	/**
	 * init() is called once, when the instance of the SOE is created.
	 */
	public void init(IServerObjectHelper soh) throws IOException, AutomationException
	{
		/*
		 * An SOE should get the Server Object from the Server Object Helper in order to make any method calls on the
		 * Server Object and release the reference after making the method calls.
		 */
		this.soHelper = soh;
		this.serverLog = getServerLogger();

		// get the Server Object (SO) this SOE is associated with
		this.mapServer = (MapServer) soHelper.getServerObject();

		// get the Map object from SO
		Map map = (Map) this.mapServer.getMap("Layers");

		// Get index of layer containing bus stations
		int busStopsLayerIndex = getLayerIndex(map, "BusStations");
		
		// contruct a feature class for bus stops
		IFeatureLayer busStationsFL = (IFeatureLayer) mapServer.getLayer("", busStopsLayerIndex);
		this.busStationsFC = new FeatureClass(busStationsFL.getFeatureClass());		
		
		int ndLayerIndex = getLayerIndex(map, "StreetsND");
		NetworkLayer busLinesNDLayer = (NetworkLayer) mapServer.getLayer("", ndLayerIndex);

		//create Network Dataset from Network Layer
		nds = new NetworkDataset(busLinesNDLayer.getNetworkDataset());

		//create route layer
		createRouteAnalysisLayer("Route");
		
		//create Stops NA Class
		INamedSet classes = naContext.getNAClasses();
		stopsNAClass = new NAClass(classes.getItemByName("Stops"));
		
		nameFieldIndex = stopsNAClass.findField("Name");
		sequenceFieldIndex = stopsNAClass.findField("Sequence");
		statusFieldIndex = stopsNAClass.findField("Status");
				
		//Retrieve Solver for the network
		naSolver = naContext.getSolver();	
		
		//Retrieve locator
		naLocator = (NALocator) naContext.getLocator();			
	}

	/**
	 * shutdown() is called once when the Server Object's context is being shut down and is about to go away.
	 */
	public void shutdown() throws IOException, AutomationException
	{
		//release all
		this.nds = null;	
		this.naSolver = null;
		this.naContext = null;
		this.stopsNAClass = null;
		this.naLocator = null;		
		
		this.busStationsFC = null;
		this.soHelper = null;
		this.serverLog = null;
	}

	/*************************************************************************************
	 * SOAP methods
	 *************************************************************************************/	
	@Override
	public BusStation[] findBusStationsNearMe(Point userLocation, double withinDistance) throws Exception
	{	
		BusStation[] busStations = null;

		try
		{
			if (userLocation != null)
			{
				// create a buffer around userLocation, using
				// withinDistance. If withinDistance is <= 0, use 500 feet
				// as default value.
				if (withinDistance < 0)
				{
					withinDistance = 1.0;//setting default distance to 1.0 mile
				}
				
				withinDistance *= 5280;//converting miles into feet

				Polygon buffer = (Polygon) userLocation.buffer(withinDistance);
				
				ArrayList<IFeature> uniqueFeatures = null;
				uniqueFeatures = getFeatures(this.busStationsFC, buffer);
				
				if(uniqueFeatures.size() == 0)
				{
					return null;
				}
				
				busStations = new BusStation[5];//uniqueFeatures.size()- restricting number of bus stations to 5
				
				Iterator<IFeature> it = uniqueFeatures.iterator();
				int i = 0;
				while(it.hasNext() && i < 5)
				{
					IFeature f = it.next();
					Point coords = (Point) f.getShape();

					BusStation busStation = new BusStation();
					busStation.setId(f.getOID());
					busStation.setCoordinates(new Coordinates(coords.getX(), coords.getY(), coords
							.getSpatialReference().getFactoryCode()));
					busStation.setRouteNumber(Integer.parseInt((String) f.getValue(3)));
					busStation.setDirection(Integer.parseInt((String) f.getValue(4)));
					busStation.setLocation((String) f.getValue(6));
					busStation.setJurisdiction((String) f.getValue(7));
					busStation.setZip(f.getValue(8).toString());

					busStations[i] = busStation;
					i++;
				}					
			}
		}
		catch(Exception e)
		{
			handleException(e, e.getCause().getClass().getName() + " occured while finding bus stops.");
			
		}

		return busStations;		
	}
	
	/**
	 * Retrieves directions
	 */
	@Override
	public String getDirections(int busStationID, Point userLocation) throws Exception
	{		
		String directionsString = "";
		
		try
		{						
			//get stops		
			Point[] points = new Point[2];
			String[] pointsDescription = new String[2];
			
			points[0] = userLocation;
			pointsDescription[0] = "My Location";
			
			//use bus id to get its point
			IFeature f = this.busStationsFC.getFeature(busStationID);
			points[1] = (Point) f.getShape();
			
			//load all stops/barriers
			Date startPoints = new Date();
			createNetworkLocation(naContext, points, "Stops");
			Date endPoints = new Date();
			this.serverLog.addMessage(0, 8000, "Loading Network Locations required -->" + (endPoints.getTime() - startPoints.getTime())/1000 + " seconds.");
					
			//solve the network
			ITrackCancel cancelTracker = new CancelTracker();		
			Date startSolve = new Date();
			naSolver.solve(naContext, new GPMessages(), cancelTracker);
			Date endSolve = new Date();
			this.serverLog.addMessage(0, 8000, "Solving Network required -->" + (endSolve.getTime() - startSolve.getTime())/1000 + " seconds.");
			
			//generate directions
			NamedSet namedSet = (NamedSet) naContext.getAgents();
			NAStreetDirectionsAgent streetAgent = (NAStreetDirectionsAgent) namedSet.getItemByName("StreetDirectionsAgent");
			streetAgent.setLengthUnits(esriUnits.esriMiles);
			streetAgent.setTimeAttributeName("Minutes");
			streetAgent.setDisplayTime(true);
			streetAgent.setAutoShowDirections(true);
			streetAgent.execute(null, null);		

			//Save as XML
			INAStreetDirectionsContainer container = streetAgent.getDirectionsContainer();
			int dirCount = container.getDirectionsCount();
			for(int i = 0; i < dirCount; i++)
			{
				INAStreetDirections directions = container.getDirections(i);
				this.serverLog.addMessage(0, 8000, "Route Name: " + directions.getRouteName() + 
						"\nRoute ID: " + directions.getRouteID() + 
						"\nDetails: ");
				for(int j = 0; j < directions.getDirectionCount(); j++)
				{
					INAStreetDirection direction = directions.getDirection(j);
					directionsString += direction.getString(0) + "\n";
				}				
			}				
		}
		catch (Exception e)
		{
			throw new Exception("Exception at NASolver.solve(): " + e.getMessage());
		}
		
		return directionsString;
	}
		
	/*************************************************************************************
	 * IRESTRequestHandler methods:  
	 *************************************************************************************/
	/**
	 * Handles REST requests. 
	 */
	@Override
	public byte[] handleRESTRequest(String capabilities, String resourceName, 
					String operationName, String operationInput, 
					String outputFormat, String requestProperties, 
					String[] responseProperties) throws IOException, AutomationException
	{
		/*
		 * This method handles REST requests by determining whether an operation
		 * or resource has been invoked and then forwards the request to
		 * appropriate methods.
		 */

		try
		{
			// if no operationName is specified send description of specified
			// resource
			if (operationName.length() == 0)
			{
				return getResource(resourceName);
			}
			else
			// invoke REST operation on specified resource
			{
				return invokeRESTOperation(capabilities, resourceName, operationName, operationInput, outputFormat,
						requestProperties, responseProperties);
			}
		}
		catch (Exception e)
		{
			return ServerUtilities.sendError(0, "Exception occurred: " + e.getMessage(), null).getBytes("utf-8");
		}
	}

	/**
	 * Returns schema of REST SOE
	 */
	@Override
	public String getSchema()
	{
		try
		{
			//main schema object
			JSONObject rootResource = new JSONObject();
			
			/*********************************************************************
			 * Root Resource
			 *********************************************************************/
			//name of root resource = name of SOE
			rootResource.put("name", "BusServices");			
			
			/*********************************************************************
			 * Root resource operations
			 *********************************************************************/
			//op array for roots 
			JSONArray rootOpsArray = new JSONArray();			
			
			//op1
			JSONObject op1 = new JSONObject();
			op1.put("name", "findBusStationNearMe");				
			JSONArray op1ParamArray = new JSONArray();
			op1ParamArray.put("userLocation");
			op1ParamArray.put("distanceInMiles");
			op1.put("parameters", op1ParamArray);
			JSONArray formats1ParamArray = new JSONArray();
			formats1ParamArray.put("json");
			op1.put("supportedOutputFormats", formats1ParamArray);

			rootOpsArray.put(op1);
			
			rootResource.put("operations", rootOpsArray);

			/*********************************************************************
			 * Sub-resources to Root resource
			 *********************************************************************/
			
			JSONArray srArray = new JSONArray();			
			/*********************************************************************
			 * Sub-resource 1 - BusStation
			 *********************************************************************/					
			JSONObject sr1 = new JSONObject();
			sr1.put("name", "BusStations");
			sr1.put("isCollection", true);
			
			//Add operations
			//op array for SR1 members 
			JSONArray op1Array = new JSONArray();			
			
			//op11
			JSONObject op11 = new JSONObject();
			op11.put("name", "getDirections");				
			JSONArray op11ParamArray = new JSONArray();
			op11ParamArray.put("userLocation");			
			op11.put("parameters", op11ParamArray);
			JSONArray formats11ParamArray = new JSONArray();
			formats11ParamArray.put("json");
			op11.put("supportedOutputFormats", formats11ParamArray);
			
			op1Array.put(op11);
								
			sr1.put("operations", op1Array);
			
			srArray.put(sr1);
			
			rootResource.put("resources", srArray);											

			return rootResource.toString();
		}
		catch (JSONException e)
		{
			e.printStackTrace();
		}
		
		return null;
	}	

	/*************************************************************************************
	 * Util methods
	 *************************************************************************************/
	/**
	 * Invokes specified REST operation on specified REST resource
	 * @param capabilitiesList
	 * @param resourceName
	 * @param operationName
	 * @param operationInput
	 * @param outputFormat
	 * @param requestProperties
	 * @param responseProperties
	 * @return
	 */
	private byte[] invokeRESTOperation(String capabilitiesList, String resourceName, String operationName,
			String operationInput, String outputFormat, String requestProperties, String[] responseProperties) throws Exception
	{
		byte[] operationOutput = null;

		JSONObject operationInputAsJSON = new JSONObject(operationInput);

		//parse request properties and create a map
		java.util.Map<String, String> requestPropertiesMap = new HashMap<String, String>();
		if (requestProperties != null && requestProperties.length() > 0)
		{
			JSONObject requestPropertiesJSON = new JSONObject(requestProperties);
			Iterator<String> jsonKeys = requestPropertiesJSON.keys();
			while (jsonKeys.hasNext())
			{
				String key = jsonKeys.next();
				requestPropertiesMap.put(key, requestPropertiesJSON.getString(key));
			}
		}

		//create a Map to hold response properties
		java.util.Map<String, String> responsePropertiesMap = new HashMap<String, String>();	
		if (resourceName.equalsIgnoreCase("") || resourceName.length() == 0)
		{	
			//check if this REST operation is authorized, by comparing against authorized capability
			if(operationName.equalsIgnoreCase("findBusStationNearMe"))
			{
				operationOutput = findBusStationsREST(operationInputAsJSON);
					
			}
			else
			{
				operationOutput = sendError(0, "Operation " + "\"" + operationName + "\" not supported on sub-resource " + resourceName + ".", 
						null).getBytes("utf-8");						
			}
		}
		else if (resourceName.matches("BusStations/[0-9]+"))
		{
			int id = Integer.valueOf(resourceName.substring(resourceName.lastIndexOf("/") + 1, resourceName.length())).intValue();
		
			if (operationName.equalsIgnoreCase("getDirections"))
			{
				operationOutput = getDirectionsREST(id, operationInputAsJSON);										
			}
			else //if unsupported operation specified by user, report error
			{
				operationOutput = sendError(0, "Operation " + "\"" + operationName + "\" not supported on sub-resource " + resourceName + ".", 
						new String[]{"no details", " specified"}).getBytes("utf-8");					
			}
		}
		else //if non existent sub-resource specified, report error
		{
			operationOutput = sendError(0, "No sub-resource by name \"" + resourceName + "\" found.", new String[]{""}).getBytes("utf-8");	
		}			
		
		//convert response properties to String array
		if (!responsePropertiesMap.isEmpty())
		{
			Set<String> keys = responsePropertiesMap.keySet();
			Iterator<String> keysIterator = keys.iterator();
			int i = 0;
			while (keysIterator.hasNext())
			{
				String key = keysIterator.next();
				String value = responsePropertiesMap.get(key);
				responseProperties[i] = key + "=" + value;
				i++;
			}
		}

		return operationOutput;
	}	
	
	/**
	 * Converts user input from JSON to Point and invokes the getDirections method
	 * @param id
	 * @param operationInput
	 * @return String - Directions from operationInput to bus station indicated by id parameter
	 */
	private byte[] getDirectionsREST(int id, JSONObject operationInput)
	{
		try
		{
			JSONObject pointJSON = operationInput.getJSONObject("userLocation"); 
			Point userLocation = new Point();
			userLocation.setX(Double.parseDouble(pointJSON.getString("x")));
			userLocation.setY(Double.parseDouble(pointJSON.getString("y")));		
			
			SpatialReferenceEnvironment sre = new SpatialReferenceEnvironment();
			userLocation.setSpatialReferenceByRef(sre.createProjectedCoordinateSystem(Integer.parseInt(pointJSON.getJSONObject("spatialReference").getString("wkid"))));
			
			String directions = getDirections(id, userLocation);
			
			JSONObject json = new JSONObject();
			json.put("directions", "directions to BusStation " + id + "\n" + directions);
			
			String returnString = json.toString();			
			
			this.serverLog.addMessage(0, 8000, "In getDirectionsOperation - 3: returnString: " + returnString);
			return returnString.getBytes("utf-8");
		}
		catch (Exception e)
		{
			handleException(new Exception(e.getCause()), e.getMessage());
		}
		
		return null;
	}
	
	/**
	 * Converts user input from JSON to Point and distance and invokes the findBusStations method
	 * @param operationInput - operation input from user, in JSON format
	 * @return String - result of findBusStation operation
	 */
	private byte[] findBusStationsREST(JSONObject operationInput)
	{
		try
		{			
			JSONObject pointJSON = operationInput.getJSONObject("userLocation"); 
			Point userLocation = new Point();
			userLocation.setX(Double.parseDouble(pointJSON.getString("x")));
			userLocation.setY(Double.parseDouble(pointJSON.getString("y")));		
			
			SpatialReferenceEnvironment sre = new SpatialReferenceEnvironment();
			userLocation.setSpatialReferenceByRef(sre.createProjectedCoordinateSystem(Integer.parseInt(pointJSON.getJSONObject("spatialReference").getString("wkid"))));
			
			this.serverLog.addMessage(0, 8000, "In findBusStationNearMe/BusStations - 2: " + 
					"userLocation: " + userLocation.getX() + ", " + userLocation.getY() + ", " + userLocation.getSpatialReference().getFactoryCode());
			
			JSONArray hubArray = new JSONArray();
			BusStation[] busStations = (BusStation[]) findBusStationsNearMe(userLocation, operationInput.getDouble("distanceInMiles"));
			
			JSONObject json = new JSONObject();
			if(busStations != null)
			{
				for(BusStation station : busStations)
				{
					//details
					json.put("details", getBusStationDescription(station.getId(), false));											
					hubArray.put(json);
				}
			}
			else
			{
				json.put("Info: ", "No Bus stations found. Try another location");
				hubArray.put(json);
			}
			
			JSONObject busStationsJSON = new JSONObject();
			busStationsJSON.put("BusStations", hubArray);
			
			String returnString = busStationsJSON.toString();			
			
			this.serverLog.addMessage(0, 8000, "In findBusStationNearMe/BusStations - 4: returnString: " + returnString);
			return returnString.getBytes("utf-8");
		}
		catch (NumberFormatException e)
		{
			e.printStackTrace();
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (JSONException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}		
		
		return null;
	}
		
	/**
	 * Returns list of features that fall within specified buffer polygon in specified feature class
	 * @param targetFC
	 * @param buffer
	 * @return ArrayList<IFeature> - extracted features
	 */
	private ArrayList<IFeature> getFeatures(FeatureClass targetFC, Polygon buffer)
	{
		ArrayList<IFeature> uniqueFeatures = null;
		
		try
		{
			QueryFilter dirQueryFilter = new QueryFilter();
			dirQueryFilter.setWhereClause("DIR = '0'");//outbound buses only

			IFeatureCursor featureCursor = targetFC.search(dirQueryFilter, false);

			/*
			 * loop through all points in bus stops FeatureCursor and see which points intersect with the
			 * bufferedStreet. collect all intersecting points in an ArrayList
			 */
			IFeature feature = featureCursor.nextFeature();
			ArrayList<IFeature> hubs = new ArrayList<IFeature>(25);
			while (feature != null)
			{
				if (!buffer.disjoint(feature.getShape()))
				{
					hubs.add(feature);
				}

				feature = featureCursor.nextFeature();
			}

			hubs.trimToSize();

			// remove duplicate entries, if any
			uniqueFeatures = new ArrayList<IFeature>(5);
			for (int i = 0; i < hubs.size(); i++)
			{
				IFeature f = hubs.get(i);
				Point p = (Point) f.getShape();
				if (i > 0)
				{
					boolean unique = false;
					for (int j = 0; j < i; j++)
					{
						Point tempPoint = (Point) hubs.get(j).getShape();
						if (p.getX() != tempPoint.getX() && p.getY() != tempPoint.getY())
						{
							unique = true;
						}
						else
						{
							unique = false;
						}
					}

					if (unique)
					{
						uniqueFeatures.add(f);
					}
				}
				else if (i == 0)
				{
					uniqueFeatures.add(f);
				}
			}

			uniqueFeatures.trimToSize();
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		
		return uniqueFeatures;
	}
	
	/**
	 * Returns description of resource specified by resourceName
	 * @param resourceName
	 * @return byte[]
	 */
	private byte[] getResource(String resourceName)
	{
		try
		{
			if(resourceName.equalsIgnoreCase("") || resourceName.length() == 0)
			{
				//this.serverLog.addMessage(1, 8000, "1");
				return getRootResourceDescription().toString().getBytes();
			}			
			else if(resourceName.equalsIgnoreCase("BusStations"))
			{
				JSONObject jobj = new JSONObject();
				jobj.put("BusStations", getBusStationCollection());
				return jobj.toString().getBytes();
			}
			else if(resourceName.matches("BusStations/[0-9]+"))
			{
				int id = Integer.valueOf(resourceName.substring(resourceName.lastIndexOf("/") + 1, resourceName.length())).intValue();
				return getBusStationDescription(id, false).toString().getBytes();
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return null;
	}	

	/**
	 * Returns description of the root resource
	 * @return description as a JSONObject
	 */
	private JSONObject getRootResourceDescription()
	{
		try
		{
			JSONObject xportService = new JSONObject();
			xportService.put("Name", "Bus Services SOE");
			xportService.put("Description", "Public Bus services using REST SOE");
			xportService.put("BusStations", getBusStationCollection());
			
			return xportService;			
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * Returns ome elements of Bus Station collection as JSON object
	 * @return JSONObject
	 */
	private JSONArray getBusStationCollection()
	{
		JSONArray busStationsArray = new JSONArray();
		for(int i = 1; i < 52; i+=10)
		{
			busStationsArray.put(getBusStationDescription(i, false));
		}		
			
		return busStationsArray;		
	}
	
	/**
	 * Returns description of a bus station indicated by specified id
	 * @param id
	 * @param requireDetails
	 * @return JSONObject
	 */
	private JSONObject getBusStationDescription(int id, boolean requireDetails)
	{
		try
		{
			if(id > 1 || id < busStationsFC.featureCount(null))
			{
				String whereClause = "\"OBJECTID\" = " + id;
				QueryFilter dirQueryFilter = new QueryFilter();
				dirQueryFilter.setWhereClause(whereClause);
				IFeatureCursor busStationsFeatureCursor = busStationsFC.search(dirQueryFilter, false);
				
				IFeature busStationFeature = busStationsFeatureCursor.nextFeature();
				JSONObject busStationJSON = new JSONObject();
				if(busStationFeature != null)
				{
					//add id and name
					busStationJSON.put("id", id);
					busStationJSON.put("name", (String) busStationFeature.getValue(6));

					Point coords = (Point) busStationFeature.getShape();					
					JSONObject coordsJSON = new JSONObject();
					coordsJSON.put("x", coords.getX());
					coordsJSON.put("y", coords.getY());
					coordsJSON.put("srid", coords.getSpatialReference().getFactoryCode());			
					busStationJSON.put("Coordinates", coordsJSON);
					
					busStationJSON.put("Direction", (String) busStationFeature.getValue(4));					
					
					if(requireDetails)
					{
						busStationJSON.put("Jurisdiction", (String) busStationFeature.getValue(7));
						busStationJSON.put("RouteNumber", Integer.parseInt((String) busStationFeature.getValue(3)));
						busStationJSON.put("Zip", ((Double) busStationFeature.getValue(8)).toString());
					}
				}

				return busStationJSON;
			}
			else
			{
				JSONObject errorObject = new JSONObject();
				errorObject.put("ErrorMessage", "ERROR: No bus stop with id = " + id + " found.");
				return errorObject;
			}
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		catch (JSONException e)
		{
			e.printStackTrace();
		}

		return null;
	}	
	
	/**
	 * Create network locations from specified array of type IPoint
	 * @param naContext NAContext
	 * @param className String
	 * @param fileName String
	 */
	private void createNetworkLocation(INAContext naContext, IPoint[] points, String classType) throws Exception
	{
		//delete existing Locations
		stopsNAClass.deleteAllRows();		

		INALocation naLocations[] = new NALocation[1];
		IPoint outPoint[] = new Point[1];
		double distanceFromPoints[] = new double[1];
		
		for(int i = 0; i < points.length; i++)
		{
			Point stopPoint = new Point();
			stopPoint.putCoords(points[i].getX(), points[i].getY());
			stopPoint.setSpatialReferenceByRef(naContext.getSpatialReference());
		
			naLocator.queryLocationByPoint(stopPoint, naLocations, outPoint, distanceFromPoints);
		
			NALocationFeature newStop = (NALocationFeature) stopsNAClass.createFeature();
			newStop.setShapeByRef(stopPoint);
			
			if(naLocations[0].isLocated())
			{
				newStop.setNALocation(naLocations[0]);
				newStop.setValue(statusFieldIndex, esriNAObjectStatus.esriNAObjectStatusOK);
			}
			else
			{
				newStop.setValue(statusFieldIndex, esriNAObjectStatus.esriNAObjectStatusElementNotLocated);
			}
			
			newStop.setValue(nameFieldIndex, "Dynamically Added Stop");
			newStop.setValue(sequenceFieldIndex, (i + 1));
			newStop.store();			
		}				
	}
	
	/**
	 * Creates NA Route Solver layer 
	 * @param layerName
	 * @return
	 */
	private INALayer createRouteAnalysisLayer(String layerName)
	{
		NALayer naLayer = null;
		try
		{
			NARouteSolver naRouteSolver = new NARouteSolver();						
			naRouteSolver.setImpedanceAttributeName("Minutes");
			naRouteSolver.setUseHierarchy(false);
			naRouteSolver.setIgnoreInvalidLocations(true);
			
			StrArray accumulators = new StrArray();
			accumulators.add("Length");
			accumulators.add("Time");
			naRouteSolver.setAccumulateAttributeNamesByRef(accumulators);
			
			naRouteSolver.setUseTimeWindows(false);
			naRouteSolver.setFindBestSequence(true);
			naRouteSolver.setPreserveFirstStop(true);
			naRouteSolver.setPreserveLastStop(true);
			naRouteSolver.setOutputLines(esriNAOutputLineType.esriNAOutputLineNone);
						
			naRouteSolver.setCreateTraversalResult(true);
			
			IDatasetComponent datasetComponent = new IDatasetComponentProxy(nds);
			DENetworkDataset deNetworkDataset = (DENetworkDataset) datasetComponent.getDataElement();
			
			naContext = (NAContext) naRouteSolver.createContext(deNetworkDataset, layerName);
			naContext.bind(nds, new GPMessages());

			naRouteSolver.updateContext(naContext, deNetworkDataset, new GPMessages());
			
			naLayer = (NALayer) naRouteSolver.createLayer(naContext);
			naLayer.setName(layerName);

		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		return naLayer;
	}		
	
	/**
	 * Retrieves ID of a layer
	 * 
	 * @param mapServer
	 * @param layerName
	 * @return
	 */
	private int getLayerIndex(Map map, String layerName) throws IOException, AutomationException
	{
		int layerID = -1;

		for (int i = 0; i < map.getLayerCount(); i++)
		{
			String name = map.getLayer(i).getName();
			if (layerName.equalsIgnoreCase(name))
			{
				layerID = i;
				break;
			}
		}

		if (layerID < 0)
		{
			//serverLog.addMessage(4, 8000, "Could not find layer " + layerName + " in " + map.getName());
		}

		return layerID;
	}

	/**
	 * Handles Exception
	 * @param e
	 * @param message
	 */
	private void handleException(Exception e, String message)
	{
		try
		{
			this.serverLog.addMessage(0, 8000, message);
			sendError(0, message, new String[]{""});
			e.printStackTrace();
		}
		catch (AutomationException e1)
		{
			e1.printStackTrace();
		}
		catch (IOException e1)
		{
			e1.printStackTrace();
		}
	}	
}